<!DOCTYPE html>
<html lang="en">
  <head>
    <title>{{ template.title }}</title>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no" />
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/4.5.3/css/bootstrap.min.css" />
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.9.4/leaflet.css" />
    <style>
      html { position: relative; min-height: 100%; }
      body { padding-top: 5rem; margin-bottom: 40px; }
      #page { padding: 1.5rem 1.5rem; }
      .navbar { border-bottom: 8px solid #3333A4; }
      .navbar-brand { font-size: 2rem; color: #3333A4 !important; text-shadow: 2px 2px 4px #ccc; }
      .json-link { font-weight: normal; }
    </style>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.6.0/jquery.min.js"></script>
    <script src="https://files.dnr.state.mn.us/lib/bootstrap4/javascripts/bootstrap.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.9.4/leaflet.js"></script>
  </head>
  <body>
    <main role="main" class="container-fluid">
      <div id="page">
